import _size from 'lodash/size';
import closest from '../../utilities/closest';
import customEvent from '../../utilities/custom-event';
import removeClass from '../../utilities/remove-class';
import hasClass from '../../utilities/has-class';
import getTransitionDurationFromElement from '../../utilities/get-transition-duration-from-element';
export class BsAlert {
    constructor() {
        this.closeEventName = 'close.bs.alert';
        this.closedEventName = 'closed.bs.alert';
    }
    handleFocusOut(event) {
        const closestDismissEl = closest(event.target, '[data-dismiss="alert"]');
        if (closestDismissEl && this.alertEl.contains(closestDismissEl)) {
            if (event.stopPropagation) {
                event.stopPropagation();
                event.preventDefault();
            }
            this.close();
        }
    }
    destroyAlert() {
        this.alertEl.parentNode.removeChild(this.alertEl);
        customEvent(this.alertEl, this.closedEventName);
    }
    close() {
        const closeEvent = customEvent(this.alertEl, this.closeEventName);
        if (closeEvent.defaultPrevented) {
            return;
        }
        removeClass(this.alertEl, 'show');
        if (!hasClass(this.alertEl, 'fade')) {
            this.destroyAlert();
            return;
        }
        const transitionDuration = getTransitionDurationFromElement(this.alertEl);
        setTimeout(() => {
            this.destroyAlert();
        }, transitionDuration);
    }
    alert(alertOptions) {
        if (_size(alertOptions) === 0) {
            return this.alertEl;
        }
        if (alertOptions === 'close') {
            this.close();
            return true;
        }
        if (typeof alertOptions === 'string') {
            throw new Error(`No method named "${alertOptions}"`);
        }
        return null;
    }
    render() {
        return (h("slot", null));
    }
    static get is() { return "bs-alert"; }
    static get properties() { return {
        "alert": {
            "method": true
        },
        "alertEl": {
            "elementRef": true
        },
        "close": {
            "method": true
        },
        "closedEventName": {
            "type": String,
            "attr": "closed-event-name"
        },
        "closeEventName": {
            "type": String,
            "attr": "close-event-name"
        }
    }; }
    static get listeners() { return [{
            "name": "click",
            "method": "handleFocusOut"
        }]; }
    static get style() { return "/**style-placeholder:bs-alert:**/"; }
}
